@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * Copyright (c) 2000, 2008 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -49,7 +49,13 @@ public BinaryExpression(Expression left, Expression right, int operator) {
 //		this.depthTracker = 1;
 //	}
 }
-
+public BinaryExpression(BinaryExpression expression) {
+	this.left = expression.left;
+	this.right = expression.right;
+	this.bits = expression.bits;
+	this.sourceStart = expression.sourceStart;
+	this.sourceEnd = expression.sourceEnd;
+}
 public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, 
 		FlowInfo flowInfo) {
 	// keep implementation in sync with CombinedBinaryExpression#analyseCode
