@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2007 IBM Corporation and others.
+ * Copyright (c) 2000, 2008 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -19,6 +19,7 @@ import org.eclipse.jdt.internal.compiler.flow.*;
 import org.eclipse.jdt.internal.compiler.lookup.*;
 import org.eclipse.jdt.internal.compiler.parser.*;
 import org.eclipse.jdt.internal.compiler.problem.*;
+import org.eclipse.jdt.internal.compiler.util.Util;
 
 public class TypeDeclaration extends Statement implements ProblemSeverities, ReferenceContext {
 	// Type decl kinds
@@ -154,7 +155,7 @@ public MethodDeclaration addMissingAbstractMethodFor(MethodBinding methodBinding
 
 	//============BINDING UPDATE==========================
 	methodDeclaration.binding = new MethodBinding(
-			methodDeclaration.modifiers, //methodDeclaration
+			methodDeclaration.modifiers | ClassFileConstants.AccSynthetic, //methodDeclaration
 			methodBinding.selector,
 			methodBinding.returnType,
 			argumentsLength == 0 ? Binding.NO_PARAMETERS : argumentTypes, //arguments bindings
@@ -398,7 +399,7 @@ public MethodBinding createDefaultConstructorWithBinding(MethodBinding inherited
 			argumentsLength == 0 ? Binding.NO_PARAMETERS : argumentTypes, //arguments bindings
 			inheritedConstructorBinding.thrownExceptions, //exceptions
 			sourceType); //declaringClass
-	
+	constructor.binding.tagBits |= (inheritedConstructorBinding.tagBits & TagBits.HasMissingType);
 	constructor.binding.modifiers |= ExtraCompilerModifiers.AccIsDefaultConstructor;
 			
 	constructor.scope = new MethodScope(this.scope, constructor, true);
@@ -782,7 +783,7 @@ public final boolean needClassInitMethod() {
 	return false;
 }
 
-public void parseMethod(Parser parser, CompilationUnitDeclaration unit) {
+public void parseMethods(Parser parser, CompilationUnitDeclaration unit) {
 	//connect method bodies
 	if (unit.ignoreMethodBodies)
 		return;
@@ -791,7 +792,7 @@ public void parseMethod(Parser parser, CompilationUnitDeclaration unit) {
 	if (this.memberTypes != null) {
 		int length = this.memberTypes.length;
 		for (int i = 0; i < length; i++)
-			this.memberTypes[i].parseMethod(parser, unit);
+			this.memberTypes[i].parseMethods(parser, unit);
 	}
 
 	//methods
@@ -938,71 +939,70 @@ public void resolve() {
 		}
 		boolean needSerialVersion = 
 						this.scope.compilerOptions().getSeverity(CompilerOptions.MissingSerialVersion) != ProblemSeverities.Ignore
-						&& sourceType.isClass() 
-						&& !sourceType.isAbstract() 
-						&& sourceType.findSuperTypeErasingTo(TypeIds.T_JavaIoExternalizable, false /*Serializable is not a class*/) == null
-						&& sourceType.findSuperTypeErasingTo(TypeIds.T_JavaIoSerializable, false /*Serializable is not a class*/) != null;
+						&& sourceType.isClass()
+						&& sourceType.findSuperTypeOriginatingFrom(TypeIds.T_JavaIoExternalizable, false /*Externalizable is not a class*/) == null
+						&& sourceType.findSuperTypeOriginatingFrom(TypeIds.T_JavaIoSerializable, false /*Serializable is not a class*/) != null;
 
 		if (needSerialVersion) {
 			// if Object writeReplace() throws java.io.ObjectStreamException is present, then no serialVersionUID is needed
 			// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=101476
-    		CompilationUnitScope compilationUnitScope = this.scope.compilationUnitScope();
+			CompilationUnitScope compilationUnitScope = this.scope.compilationUnitScope();
 			MethodBinding methodBinding = sourceType.getExactMethod(TypeConstants.WRITEREPLACE, new TypeBinding[0], compilationUnitScope);
-   			ReferenceBinding[] throwsExceptions;
+			ReferenceBinding[] throwsExceptions;
 			needSerialVersion = 
 				methodBinding == null
-    				|| !methodBinding.isValidBinding()
-    				|| methodBinding.returnType.id != TypeIds.T_JavaLangObject
-    				|| (throwsExceptions = methodBinding.thrownExceptions).length != 1
-    				|| throwsExceptions[0].id != TypeIds.T_JavaIoObjectStreamException;
-    		if (needSerialVersion) {
-    			// check the presence of an implementation of the methods
-    			// private void writeObject(java.io.ObjectOutputStream out) throws IOException
-    			// private void readObject(java.io.ObjectInputStream out) throws IOException
-    			boolean hasWriteObjectMethod = false;
-    			boolean hasReadObjectMethod = false;
-    			TypeBinding argumentTypeBinding = this.scope.getType(TypeConstants.JAVA_IO_OBJECTOUTPUTSTREAM, 3);
-     			if (argumentTypeBinding.isValidBinding()) {
-            		methodBinding = sourceType.getExactMethod(TypeConstants.WRITEOBJECT, new TypeBinding[] { argumentTypeBinding }, compilationUnitScope);
-            		hasWriteObjectMethod = methodBinding != null
-            				&& methodBinding.isValidBinding()
-            				&& methodBinding.modifiers == ClassFileConstants.AccPrivate
-            				&& methodBinding.returnType == TypeBinding.VOID
-            				&& (throwsExceptions = methodBinding.thrownExceptions).length == 1
-            				&& throwsExceptions[0].id == TypeIds.T_JavaIoException;
-    			}
-    			argumentTypeBinding = this.scope.getType(TypeConstants.JAVA_IO_OBJECTINPUTSTREAM, 3);
-     			if (argumentTypeBinding.isValidBinding()) {
-            		methodBinding = sourceType.getExactMethod(TypeConstants.READOBJECT, new TypeBinding[] { argumentTypeBinding }, compilationUnitScope);
-            		hasReadObjectMethod = methodBinding != null
-            				&& methodBinding.isValidBinding()
-            				&& methodBinding.modifiers == ClassFileConstants.AccPrivate
-            				&& methodBinding.returnType == TypeBinding.VOID
-            				&& (throwsExceptions = methodBinding.thrownExceptions).length == 1
-            				&& throwsExceptions[0].id == TypeIds.T_JavaIoException;
-    			}
-    			needSerialVersion = !hasWriteObjectMethod || !hasReadObjectMethod;
-    		}
+					|| !methodBinding.isValidBinding()
+					|| methodBinding.returnType.id != TypeIds.T_JavaLangObject
+					|| (throwsExceptions = methodBinding.thrownExceptions).length != 1
+					|| throwsExceptions[0].id != TypeIds.T_JavaIoObjectStreamException;
+			if (needSerialVersion) {
+				// check the presence of an implementation of the methods
+				// private void writeObject(java.io.ObjectOutputStream out) throws IOException
+				// private void readObject(java.io.ObjectInputStream out) throws IOException
+				boolean hasWriteObjectMethod = false;
+				boolean hasReadObjectMethod = false;
+				TypeBinding argumentTypeBinding = this.scope.getType(TypeConstants.JAVA_IO_OBJECTOUTPUTSTREAM, 3);
+				if (argumentTypeBinding.isValidBinding()) {
+					methodBinding = sourceType.getExactMethod(TypeConstants.WRITEOBJECT, new TypeBinding[] { argumentTypeBinding }, compilationUnitScope);
+					hasWriteObjectMethod = methodBinding != null
+							&& methodBinding.isValidBinding()
+							&& methodBinding.modifiers == ClassFileConstants.AccPrivate
+							&& methodBinding.returnType == TypeBinding.VOID
+							&& (throwsExceptions = methodBinding.thrownExceptions).length == 1
+							&& throwsExceptions[0].id == TypeIds.T_JavaIoException;
+				}
+				argumentTypeBinding = this.scope.getType(TypeConstants.JAVA_IO_OBJECTINPUTSTREAM, 3);
+				if (argumentTypeBinding.isValidBinding()) {
+					methodBinding = sourceType.getExactMethod(TypeConstants.READOBJECT, new TypeBinding[] { argumentTypeBinding }, compilationUnitScope);
+					hasReadObjectMethod = methodBinding != null
+							&& methodBinding.isValidBinding()
+							&& methodBinding.modifiers == ClassFileConstants.AccPrivate
+							&& methodBinding.returnType == TypeBinding.VOID
+							&& (throwsExceptions = methodBinding.thrownExceptions).length == 1
+							&& throwsExceptions[0].id == TypeIds.T_JavaIoException;
+				}
+				needSerialVersion = !hasWriteObjectMethod || !hasReadObjectMethod;
+			}
 		}
 		// generics (and non static generic members) cannot extend Throwable
-		if (sourceType.findSuperTypeErasingTo(TypeIds.T_JavaLangThrowable, true) != null) {
+		if (sourceType.findSuperTypeOriginatingFrom(TypeIds.T_JavaLangThrowable, true) != null) {
 			ReferenceBinding current = sourceType;
 			checkEnclosedInGeneric : do {
 				if (current.isGenericType()) {
 					this.scope.problemReporter().genericTypeCannotExtendThrowable(this);
-					break checkEnclosedInGeneric;						
+					break checkEnclosedInGeneric;
 				}
 				if (current.isStatic()) break checkEnclosedInGeneric;
 				if (current.isLocalType()) {
 					NestedTypeBinding nestedType = (NestedTypeBinding) current.erasure();
 					if (nestedType.scope.methodScope().isStatic) break checkEnclosedInGeneric;
-				}				
+				}
 			} while ((current = current.enclosingType()) != null);
 		}
 		this.maxFieldCount = 0;
 		int lastVisibleFieldID = -1;
 		boolean hasEnumConstants = false;
-		boolean hasEnumConstantsWithoutBody = false;
+		FieldDeclaration[] enumConstantsWithoutBody = null;
 		
 		if (this.typeParameters != null) {
 			for (int i = 0, count = this.typeParameters.length; i < count; i++) {
@@ -1020,8 +1020,11 @@ public void resolve() {
 				switch(field.getKind()) {
 					case AbstractVariableDeclaration.ENUM_CONSTANT:
 						hasEnumConstants = true;
-						if (!(field.initialization instanceof QualifiedAllocationExpression))
-							hasEnumConstantsWithoutBody = true;
+						if (!(field.initialization instanceof QualifiedAllocationExpression)) {
+							if (enumConstantsWithoutBody == null)
+								enumConstantsWithoutBody = new FieldDeclaration[count];
+							enumConstantsWithoutBody[i] = field;
+						}
 					case AbstractVariableDeclaration.FIELD:
 						FieldBinding fieldBinding = field.binding;
 						if (fieldBinding == null) {
@@ -1063,11 +1066,19 @@ public void resolve() {
 			case TypeDeclaration.ENUM_DECL :
 				// check enum abstract methods
 				if (this.binding.isAbstract()) {
-					if (!hasEnumConstants || hasEnumConstantsWithoutBody) {
+					if (!hasEnumConstants) {
+						for (int i = 0, count = this.methods.length; i < count; i++) {
+							final AbstractMethodDeclaration methodDeclaration = this.methods[i];
+							if (methodDeclaration.isAbstract() && methodDeclaration.binding != null)
+								this.scope.problemReporter().enumAbstractMethodMustBeImplemented(methodDeclaration);
+						}
+					} else if (enumConstantsWithoutBody != null) {
 						for (int i = 0, count = this.methods.length; i < count; i++) {
 							final AbstractMethodDeclaration methodDeclaration = this.methods[i];
 							if (methodDeclaration.isAbstract() && methodDeclaration.binding != null) {
-								this.scope.problemReporter().enumAbstractMethodMustBeImplemented(methodDeclaration);
+								for (int f = 0, l = enumConstantsWithoutBody.length; f < l; f++)
+									if (enumConstantsWithoutBody[f] != null)
+										this.scope.problemReporter().enumConstantMustImplementAbstractMethod(methodDeclaration, enumConstantsWithoutBody[f]);
 							}
 						}
 					}
@@ -1091,10 +1102,19 @@ public void resolve() {
 				// if the type is package-info, the javadoc was resolved as part of the compilation unit javadoc
 				this.javadoc.resolve(this.scope);
 			}
-		} else if (sourceType != null && !sourceType.isLocalType()) {
-			this.scope.problemReporter().javadocMissing(this.sourceStart, this.sourceEnd, sourceType.modifiers);
+		} else if (!sourceType.isLocalType()) {
+			// Set javadoc visibility
+			int visibility = sourceType.modifiers & ExtraCompilerModifiers.AccVisibilityMASK;
+			ProblemReporter reporter = this.scope.problemReporter();
+			int severity = reporter.computeSeverity(IProblem.JavadocMissing);
+			if (severity != ProblemSeverities.Ignore) {
+				if (this.enclosingType != null) {
+					visibility = Util.computeOuterMostVisibility(enclosingType, visibility);
+				}
+				int javadocModifiers = (this.binding.modifiers & ~ExtraCompilerModifiers.AccVisibilityMASK) | visibility;
+				reporter.javadocMissing(this.sourceStart, this.sourceEnd, severity, javadocModifiers);
+			}
 		}
-		
 	} catch (AbortType e) {
 		this.ignoreFurtherInvestigation = true;
 		return;
@@ -1174,9 +1194,6 @@ public void tagAsHavingErrors() {
  *
  */
 public void traverse(ASTVisitor visitor, CompilationUnitScope unitScope) {
-
-	if (this.ignoreFurtherInvestigation)
-		return;
 	try {
 		if (visitor.visit(this, unitScope)) {
 			if (this.javadoc != null) {
@@ -1232,8 +1249,6 @@ public void traverse(ASTVisitor visitor, CompilationUnitScope unitScope) {
  *	Iteration for a local innertype
  */
 public void traverse(ASTVisitor visitor, BlockScope blockScope) {
-	if (this.ignoreFurtherInvestigation)
-		return;
 	try {
 		if (visitor.visit(this, blockScope)) {
 			if (this.javadoc != null) {
@@ -1290,8 +1305,6 @@ public void traverse(ASTVisitor visitor, BlockScope blockScope) {
  *
  */
 public void traverse(ASTVisitor visitor, ClassScope classScope) {
-	if (this.ignoreFurtherInvestigation)
-		return;
 	try {
 		if (visitor.visit(this, classScope)) {
 			if (this.javadoc != null) {
